<?php
namespace Datec\DatecBooking\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Datec\DatecBooking\Domain\Model\Reservation;
use Datec\DatecBooking\Domain\Repository\ReservationRepository;
use Datec\DatecBooking\Domain\Repository\ResourceRepository;
use Datec\DatecBooking\Services\MailService;
use Datec\DatecContact\Domain\Model\Contact;
use Datec\DatecContact\Domain\Repository\ContactRepository;
use TYPO3\CMS\Core\Messaging\FlashMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Domain\Model\FrontendUser;
use TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException;
use TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * @package datec_booking
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class BookingController extends ActionController {
	
	/**
	 * $extKey
	 */
	protected $extKey;
	
	/**
	 * $feUser
	 */
	protected $feUser;
	
	/**
	 * @var Contact $contact
	 */
	protected $contact;
	
	/**
	 * @var Reservation $reservation
	 */
	protected $reservation;
	
	/**
	 * $contactRepository
	 *
	 * @var ContactRepository
	 * */
	protected $contactRepository;
	
	/**
	 * $reservationRepository
	 *
	 * @var ReservationRepository
	 */
	protected $reservationRepository;
	
	/**
	 * $resourceRepository
	 *
	 * @var ResourceRepository
	 */
	protected $resourceRepository;
	
	/**
	 * $mailService
	 *
	 * @var MailService
	 */
	protected $mailService;
	
	/**
	 * $feUserRepository
	 *
	 * @var FrontendUserRepository
	 */
	protected $feUserRepository;
	
	/**
	 * $persistenceManager
	 *
	 * @var PersistenceManager
	 */
	protected $persistenceManager;

    /**
     * @param ContactRepository $contactRepository
     */
    public function injectContactRepository(ContactRepository $contactRepository)
    {
        $this->contactRepository = $contactRepository;
    }

    /**
     * @param FrontendUserRepository $feUserRepository
     */
    public function injectFeUserRepository(FrontendUserRepository $feUserRepository)
    {
        $this->feUserRepository = $feUserRepository;
    }

    /**
     * @param MailService $mailService
     */
    public function injectMailService(MailService $mailService)
    {
        $this->mailService = $mailService;
    }

    /**
     * @param PersistenceManager $persistenceManager
     */
    public function injectPersistenceManager(PersistenceManager $persistenceManager)
    {
        $this->persistenceManager = $persistenceManager;
    }

    /**
     * @param ReservationRepository $reservationRepository
     */
    public function injectReservationRepository(ReservationRepository $reservationRepository)
    {
        $this->reservationRepository = $reservationRepository;
    }

    /**
     * @param ResourceRepository $resourceRepository
     */
    public function injectResourceRepository(ResourceRepository $resourceRepository)
    {
        $this->resourceRepository = $resourceRepository;
    }
	
	/**
	 * initialize current action
	 * @return void
	 */
	public function initializeAction() {
		$this->extKey = $this->request->getControllerExtensionKey();
		$this->feUser = $this->getFeUser();
		
	}
	
	/**
	 * action showResources
	 *
	 * @return void
	 */
	public function showResourcesAction() {
		$this->view->assign('pageId', $GLOBALS['TSFE']->id);
		$this->view->assign('settings', $this->settings);
	}
	
	/**
	 * action listResources
	 *
	 * @return void
	 */
	public function listResourcesAction() {
		$resourcesResult = $this->resourceRepository->findByPage($GLOBALS['TSFE']->id);
		if ($resourcesResult) {
			$resourcesResult = $resourcesResult->toArray();
			if (count($resourcesResult)) {
				$reservationEvents = array();
				$this->view->assign('resources', $resourcesResult);				

				/* @var \Datec\DatecBooking\Domain\Model\Resource $resource  */
				foreach ($resourcesResult as $resource) {
					$reservationsResult = $this->reservationRepository->findByResource($resource->getUid());					
					if ($reservationsResult) {
						$reservationsResult = $reservationsResult->toArray();
						if (count($reservationsResult)) {						
							foreach ($reservationsResult as $reservation) {
								$reservationEvents[$resource->getUid()][] = $this->generateReservationEventObject($reservation);
							}													
						}
					}
					$reservationEvents[$resource->getUid()] = json_encode($reservationEvents[$resource->getUid()]);
				}
				$this->view->assign('reservationEvents', $reservationEvents);
			} else {
				$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.messages.noResources',$this->extKey), '', FlashMessage::WARNING);
			}
		} else {
			$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.errors.dbError',$this->extKey), '', FlashMessage::ERROR);
		}

		$this->view->assign('settings', $this->settings);
	}
	
	/**
	 * action newReservationForm
	 *
	 * @return void
	 */
	public function newReservationFormAction() {	
		if ($this->request->hasArgument('resourceId')) {
			$resourceId = $this->request->getArgument('resourceId');
			
			$resourceResult = $this->resourceRepository->findByUid($resourceId);
			if ($resourceResult) {	
				$this->initializeReservation($resourceResult);
				$formFields = $this->getFormFields();
				
				$this->view->assign('duplicant1', md5(GeneralUtility::generateRandomBytes(64)));
				$this->view->assign('formFields', $formFields);
				$this->view->assign('reservation', $this->reservation);	
				$this->view->assign('lang', $GLOBALS['TSFE']->lang);
			} else {
				$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.errors.dbError',$this->extKey), '', FlashMessage::ERROR);
			}
		} else {
			$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.errors.noResourceId',$this->extKey), '', FlashMessage::ERROR);
		}
			
		$this->view->assign('settings', $this->settings);
	}

    /**
     * action createReservation
     *
     * @return void
     * @throws IllegalObjectTypeException
     */
	public function createReservationAction() {
		$this->initializeReservation();
		$success = TRUE;
		
		if ($this->request->hasArgument('duplicant1') && $this->request->hasArgument('duplicant2')) { // spam protection values, they can only match when form submitted via 'DatecContact.createGuestbookEntry()'
			$duplicant1 = $this->request->getArgument('duplicant1');
			$duplicant2 = $this->request->getArgument('duplicant2');		
			
			if ($duplicant1 == $duplicant2) {
				
				if ($this->reservation->getOccupancyFrom() > time()) { // is reservation in future?
					$occupationResult = $this->reservationRepository->findByOccupation($this->reservation);
						
					if ($occupationResult) {
						$occupationResult = $occupationResult->toArray();
						if (count($occupationResult) <= 0) {
							if ($this->contact) {
								if (!$this->contact->getFeUser()) { // has no frontend user assigned? is public contact
									$contactResult = $this->contactRepository->findByCredentials($this->contact->getEmail());
					
									if (is_object($contactResult)) {
										// we know this public user
										if ($contactResult->isBlocked()) {
											// that badboy is not allowed here
											$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.messages.contact.blocked_true',$this->extKey), '', FlashMessage::WARNING);
											$success = FALSE;
										} else {
											// he's a'ight, use him
											$this->contact = $contactResult;
										}
									} else if (empty($contactResult)) {
										// we don't know this public user, dont use the result, save a new
			 							$this->contactRepository->add($this->contact);
										$this->persistenceManager->persistAll();
									} else {
										$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.errors.dbError',$this->extKey), '', FlashMessage::ERROR);
										$success = FALSE;
									}
								}
							}
							
			
							if ($success) { // no problems with the contact?
								$this->reservation->setContact($this->contact);
									
								$this->reservationRepository->add($this->reservation);
								$this->persistenceManager->persistAll();
								$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.messages.createReservation_success',$this->extKey), '', FlashMessage::OK);
							
								$formFields = $this->getFormFields();
								$htmlMailUser = $this->mailService->generateContactUserMail($this->contact, $this->reservation, $formFields);
								$this->mailService->sendExternMail(LocalizationUtility::translate('tx_datecbooking.mails.subjects.reservationUserMail', $this->extKey), $htmlMailUser, '', $this->contact->getEmail(), $this->contact->getFirstName()." ".$this->contact->getLastName());
								// admin mail
								if ($this->settings['mail']['internMailTo']) {
									$htmlMailAdmin = $this->mailService->generateContactAdminMail($this->contact, $this->reservation, $formFields);
									$this->mailService->sendInternMail(LocalizationUtility::translate('tx_datecbooking.mails.subjects.reservationAdminMail', $this->extKey), $htmlMailAdmin);
								}
							}
						} else {
							$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.messages.noVacancy',$this->extKey), '', FlashMessage::WARNING);
						}
					} else {
						$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.errors.dbError',$this->extKey), '', FlashMessage::ERROR);
					}
				} else {
					$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.messages.notInFuture',$this->extKey), '', FlashMessage::WARNING);
				}	
			} else {
				$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.messages.spamProtection_failed',$this->extKey), '', FlashMessage::ERROR);
			}
		} else {
			$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.messages.spamProtection_failed',$this->extKey), '', FlashMessage::ERROR);
		}

		$this->view->assign('reservation', $this->reservation);
		$this->view->assign('settings', $this->settings);
	}
	
	/**
	 * look for current frontend user
	 *
	 * @return array|boolean $feUser or boolean FALSE if no user found
	 */
	private function getFeUser() {
		if(isset($GLOBALS['TSFE']->loginUser)) {
			return $GLOBALS['TSFE']->fe_user->user;
		}
	
		return FALSE;
	}
	
	/**
	 * use TS configuration to determine used form fields
	 *
	 * @return array $formFields
	 */
	private function getFormFields() {
        $formFields = [];
		
		$formFieldsConfiguration = explode(',',$this->settings['reservations']['formFields']);
		$requiredFormFieldsConfiguration = explode(',',$this->settings['reservations']['requiredFormFields']);
		
		if (count($formFieldsConfiguration)) {
			foreach ($formFieldsConfiguration as $propertyNameRaw) {
				$propertyName = GeneralUtility::underscoredToLowerCamelCase($propertyNameRaw);

				if ($this->contact->_hasProperty($propertyName)) {
					$formFields[$propertyName]['propertyName'] = $propertyName;
					$formFields[$propertyName]['propertyNameRaw'] = $propertyNameRaw;
					$formFields[$propertyName]['required'] = in_array($propertyNameRaw, $requiredFormFieldsConfiguration);
				}
			}
		}
		
		if (!in_array('email', array_keys($formFields))) {
			$this->addFlashMessage(LocalizationUtility::translate('tx_datecbooking.errors.configurationError',$this->extKey), '', FlashMessage::ERROR);
			unset($formFields);
		}

		return $formFields;
	}
	
	/**
	 * initialize a new reservation object and fill it with form data when provided,
	 * also sets the assigned contact (creator of message) and resource
	 * 
	 * @param \Datec\DatecBooking\Domain\Model\Resource $resource refer to this this resource, only needed on initial creation
	 * @return void
	 */
	private function initializeReservation ($resource = NULL) {
		$this->reservation = new Reservation();
		$this->contact = new Contact();
		
		$this->reservation->setPid($this->settings['storagePid']);
		$this->contact->setPid($this->settings['contactsStoragePid']);
		
		if ($this->feUser) {
			$contactResult = $this->contactRepository->findByFeUser($this->feUser['uid']);
			
			if (is_object($contactResult)) {
				$this->contact = $contactResult;
			} else if (empty($contactResult)) {
				// none found, use this fe user as new creator
                /* @var FrontendUser $feUserResult */
				$feUserResult = $this->feUserRepository->findByUid($this->feUser['uid']);
				if ($feUserResult) {
					$this->contact->setFeUser($feUserResult);
				}				
			}
		}	
		
		if ($this->request->hasArgument('reservation')) {
			$formDataReservation = (array) $this->request->getArgument('reservation');

			foreach ($formDataReservation as $propertyName => $propertyValue) {
				if ($propertyName == 'resource') {
				    /* @var \Datec\DatecBooking\Domain\Model\Resource $resourceResult */
					$resourceResult = $this->resourceRepository->findByUid($propertyValue['__identity']);
					if ($resourceResult) {					
						$this->reservation->setResource($resourceResult);
						
						if ($resourceResult->isAutoConfirmReservations()) {
							$this->reservation->setConfirmed(TRUE);
						} else {
							$this->reservation->setConfirmed(FALSE);
						}
					}
				} else if ($propertyName == 'occupancyFrom') {
					$this->reservation->_setProperty($propertyName, strtotime(str_replace('.', '-', $propertyValue)));
				} else if ($propertyName == 'occupancyTo') {
					$this->reservation->_setProperty($propertyName, strtotime(str_replace('.', '-', $propertyValue) . ' 23:59:59'));
				} else {
					$this->reservation->_setProperty($propertyName, $propertyValue);
				}
			}		
		}
		
		if (isset($_GET['tx_dateccontact_contact']['contact'])) {
			$formDataContact = GeneralUtility::_GP(['tx_dateccontact_contact']['contact']);
				
			foreach ($formDataContact as $propertyName => $propertyValue) {
				$this->contact->_setProperty($propertyName, $propertyValue);
			}
		}
		
		$this->reservation->setContact($this->contact);
		if (isset($resource)) {
			$this->reservation->setResource($resource);
		}
	}

    /**
     * creates a event object for the given reservation
     *
     * @param Reservation $reservation
     * @return \stdClass
     */
	private function generateReservationEventObject($reservation) {
		$event = new \stdClass();
		$startDate = new \DateTime();
		$endDate = new \DateTime();
		
		$startDate->setTimestamp($reservation->getOccupancyFrom());
		$endDate->setTimestamp($reservation->getOccupancyTo());
		
		if ($endDate > $startDate) {
			$title = $startDate->format($this->settings['display']['dateFormat'])." - ".$endDate->format($this->settings['display']['dateFormat']);
		} else {
			$title = $startDate->format($this->settings['display']['dateFormat']);
		}
		
		$event->id = $reservation->getUid();
		$event->title = $title;
		$event->start = date('c', $reservation->getOccupancyFrom());
		$event->end = date('c', $reservation->getOccupancyTo());
		
		return $event;
	}
	
}