<?php
namespace Datec\DatecBooking\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Datec\DatecBooking\Domain\Model\Reservation;
use TYPO3\CMS\Extbase\Persistence\QueryInterface;
use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;
use TYPO3\CMS\Extbase\Persistence\Repository;

/**
 * @package datec_booking
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class ReservationRepository extends Repository {
	
	/**
	 * PR: this is not ideal yet, because the query funtions do not support conditional sorting,
	 * that means either build cutom querries or have this slightly off output:
	 * the current sorting can place posts published (starttime) before created (crdate) posts on the same day, even if the created one is newer by time
	 */
	protected $defaultOrderings = array(
		'crdate' => QueryInterface::ORDER_DESCENDING
	 );
	
	/**
	 * overrides default settings initialisation
	 * - we do not want to restrict the backend storage pages
	 */
	 public function initializeObject() {
	 	$query = $this->createQuery();	 	
        $defaultQuerySettings = $query->getQuerySettings();
        $defaultQuerySettings->setRespectStoragePage(FALSE);
        $this->setDefaultQuerySettings($defaultQuerySettings);
        unset($query);
	}
	
	/**
	 * Finds all objects
	 *
	 * @param boolean $ingoreEnableFields ignore enable fields in search
	 * @param array $enableFieldsToBeIgnored array of columns to ignore when $ignoreEnableFields == TRUE
	 * @param boolean $includeDeleted include deleted in search, defaults to FALSE  
	 * @param array $newOrder array of 'COLUMNNAME' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_*
	 * @return object The matching object if found, otherwise NULL
	 */
	public function findAll($ingoreEnableFields = FALSE, $enableFieldsToBeIgnored = array(), $includeDeleted = FALSE, $newOrder = array()) {
		$query = $this->createQuery();
		$defaultQuerySettings = $query->getQuerySettings();
		$defaultQuerySettings->setIgnoreEnableFields($ingoreEnableFields);
		$defaultQuerySettings->setIncludeDeleted($includeDeleted);
		$defaultQuerySettings->setEnableFieldsToBeIgnored($enableFieldsToBeIgnored);
		if (!empty($newOrder) && is_array($newOrder)) {
			$this->defaultOrderings = $newOrder;
		}
		$this->setDefaultQuerySettings($defaultQuerySettings);
		return $query->execute();
	}
	
	/**
	 * Finds an object matching the given identifier.
	 *
	 * @param int $uid The identifier of the object to find
	 * @param boolean $ingoreEnableFields ignore enable fields in search
	 * @param array $enableFieldsToBeIgnored array of columns to ignore when $ignoreEnableFields == TRUE
	 * @param boolean $includeDeleted include deleted in search, defaults to FALSE 
	 * @return object The matching object if found, otherwise NULL
	 */
	public function findByUid($uid, $ingoreEnableFields = FALSE, $enableFieldsToBeIgnored = array(), $includeDeleted = FALSE) {
		$query = $this->createQuery();
		$defaultQuerySettings = $query->getQuerySettings();
		$defaultQuerySettings->setIgnoreEnableFields($ingoreEnableFields);
		$defaultQuerySettings->setIncludeDeleted($includeDeleted);
		$defaultQuerySettings->setEnableFieldsToBeIgnored($enableFieldsToBeIgnored);
		$this->setDefaultQuerySettings($defaultQuerySettings);
		$object = $query->matching($query->equals('uid', $uid))->execute()->getFirst();
	
		return $object;
	}
	
	/**
	 * Finds all objects by given resource identifier
	 *
	 * @param int $resourceId The identifier of the given resource to search by
	 * @param boolean $ingoreEnableFields ignore enable fields in search
	 * @param array $enableFieldsToBeIgnored array of columns to ignore when $ignoreEnableFields == TRUE
	 * @param boolean $includeDeleted include deleted in search, defaults to FALSE 
	 * @return object The matching object if found, otherwise NULL
	 */
	public function findByResource($resourceId, $ingoreEnableFields = FALSE, $enableFieldsToBeIgnored = array(), $includeDeleted = FALSE) {
		$query = $this->createQuery();
		$defaultQuerySettings = $query->getQuerySettings();
		$defaultQuerySettings->setIgnoreEnableFields($ingoreEnableFields);
		$defaultQuerySettings->setIncludeDeleted($includeDeleted);
		$defaultQuerySettings->setEnableFieldsToBeIgnored($enableFieldsToBeIgnored);
		$this->setDefaultQuerySettings($defaultQuerySettings);
		$object = $query->matching($query->equals('resource', $resourceId))->execute();
	
		return $object;
	}
	
	/**
	 * used to find iff the ressource is already occupied
	 * 
	 * @param Reservation $reservation new reservation
	 * @return array|QueryResultInterface
	 */
	public function findByOccupation($reservation) {
		$constraints = array();
		$query = $this->createQuery();
		
		$constraints[] = $query->equals('resource', $reservation->getResource());
		
		$from = $reservation->getOccupancyFrom();
		$to = $reservation->getOccupancyTo();
		if (!empty($from) && !empty($to)) {							
			$datetimeConstraints[] = $query->logicalAnd($query->greaterThanOrEqual('occupancyFrom', $from), $query->lessThanOrEqual('occupancyFrom', $to));			
			$datetimeConstraints[] = $query->logicalAnd($query->greaterThanOrEqual('occupancyTo', $from), $query->lessThanOrEqual('occupancyTo', $to));
			$datetimeConstraints[] = $query->logicalAnd($query->lessThanOrEqual('occupancyFrom', $from), $query->greaterThanOrEqual('occupancyTo', $to));
			
			$constraints[] = $query->logicalOr($datetimeConstraints);
		}
		
		return $query->matching($query->logicalAnd($constraints))->execute();		
		
	}
	
}