<?php
namespace Datec\DatecBooking\Services;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Mail\MailMessage;
use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManager;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 *
 * @package datec_booking
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class MailService implements SingletonInterface  {
	
	/**
	 * $configurationManager
	 * 
	 * @var ConfigurationManager
	 * */
	protected $configurationManager;

    /**
     * @param ConfigurationManager $configurationManager
     */
    public function injectConfigurationManager(ConfigurationManager $configurationManager)
    {
        $this->configurationManager = $configurationManager;
    }
	
	/**
	 * Sends mail with message to an intern recipient.
	 * 
	 * @param string $subject Subject of message
	 * @param string $html HTML message text
	 * @param string $plain Plain message text
	 * 
	 * @return boolean Message was send
	 */
	public function sendInternMail($subject, $html, $plain = '') {
		$settings = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
		$message = new MailMessage();
		
		$message->setFrom(array($settings['mail']['internMailFrom'] => $settings['mail']['internMailFromName']));
		$message->setTo(array($settings['mail']['internMailTo'] => ''));
		
		$message->setSubject($subject);
		$message->setBody($html, 'text/html', 'utf-8');
		if ($plain) {
			$message->addPart($plain, 'text/plain', 'utf-8');
		}
		
		$message->send();
		if($message->isSent()) {
			return TRUE;
		}
	
		return FALSE;
	}
	
	/**
	 * Sends mail with message to an extern recipients.
	 *
	 * @param string $subject Subject of message
	 * @param string $html HTML message text
	 * @param string $plain Plain message text
	 * @param string $recipientMail Email address of recipient
	 * @param string $recipientName Name of recipient
	 *
	 * @return boolean Message was send
	 */
	public function sendExternMail($subject, $html, $plain = '', $recipientMail = '', $recipientName = '') {
		$settings = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
		$message = new MailMessage();
		
		$message->setFrom(array($settings['mail']['internMailFrom'] => $settings['mail']['internMailFromName']));
		$message->setTo(array($recipientMail => $recipientName));
		
		$message->setSubject($subject);
		$message->setBody($html, 'text/html', 'utf-8');
		if ($plain) {
			$message->addPart($plain, 'text/plain', 'utf-8');
		}
	
		$message->send();
		if($message->isSent()) {
			return TRUE;
		}
	
		return FALSE;
	}
	
	/**
	 * @param object $user
	 * @param object $reservation
	 * @param array $formFields
	 * @return string
	 */
	public function generateContactUserMail($user, $reservation, $formFields) {
		$emailView = new StandaloneView();
		$templateName = 'Email/ContactUserMail.html';		
		$settings = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
		$extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $emailView->setTemplate($templateName);
        $emailView->getRequest()->setControllerExtensionName('DatecBooking');
        $emailView->setTemplateRootPaths($extbaseFrameworkConfiguration['view']['templateRootPaths']);

		$fullURL =  GeneralUtility::getIndpEnv('TYPO3_SITE_URL');
        $logoDefault = "typo3conf/ext/datec_contact/Resources/Public/Images/logo.png";
        $logoPath = $settings['mail']['logoInMail'];

        $logoPath = (strstr($logoPath,"www") || strstr($logoPath,"http")) ? $logoPath : $fullURL.$logoPath;
        $logoUrl = (GeneralUtility::getUrl($logoPath) !== FALSE) ? $logoPath : $fullURL.$logoDefault;
		
		if (is_object($user)) {
			switch ($settings['display']['usernames']) {
				case 'username' :$userName = $user->getUsername(); break;
				case 'realname' : $userName = $user->getFirstName()." ".$user->getLastName; break;
				case 'realnameFull' : $userName = $user->getTitle." ".$user->getFirstName()." ".$user->getMiddleName()." ".$user->getLastName; break;
				default : $userName = $user->getUsername();
			}
		}
		
		$baseInfoArr = array(
			'baseURL' 	=> $fullURL,
			'userName' 	=> $userName,
			'logoUrl' 	=> $logoUrl,
			'logoLink'	=> empty($settings['mail']['logoLink']) ? $fullURL : $settings['mail']['logoLink']
		);
		
		$propertiesArr = array();
		foreach ($formFields as $propertyName => $formField) {
			if (is_object($user)) {
				$value = ($user->_hasProperty($propertyName)) ? $user->_getProperty($propertyName) : '';	
				if(!empty($value)){	
					$propertiesArr['properties'][] = array(
						'name' => $formField['propertyNameRaw'],
						'value' => $value
					);
				}
			}
		}
		
		if (is_object($reservation)) {
			$emailView->assign('reservation', $reservation);
		}
		
		$emailView->assignMultiple($propertiesArr);
		$emailView->assignMultiple($baseInfoArr);
		$emailView->assign('settings', $settings);
		
		$emailBody = $emailView->render();
		
		return $emailBody;
	}
	
	/**
	 * @param object $user
	 * @param object $reservation
	 * @param array $formFields
	 * @return string
	 */
	public function generateContactAdminMail($user, $reservation, $formFields) {
		$emailView = new StandaloneView();
		$templateName = 'Email/ContactAdminMail.html';
		$settings = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
		$extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $emailView->setTemplate($templateName);
        $emailView->getRequest()->setControllerExtensionName('DatecBooking');
        $emailView->setTemplateRootPaths($extbaseFrameworkConfiguration['view']['templateRootPaths']);
	
		$propertiesArr = array();
		foreach ($formFields as $propertyName => $formField) {
			if (is_object($user)) {
				$value = ($user->_hasProperty($propertyName)) ? $user->_getProperty($propertyName) : '';
				if(!empty($value)){
					$propertiesArr['properties'][] = array(
						'name' => $formField['propertyNameRaw'],
						'value' => $value
					);
				}
			}
		}
	
		if (is_object($reservation)) {
			$emailView->assign('reservation', $reservation);
		}
	
		$emailView->assignMultiple($propertiesArr);
		$emailView->assign('settings', $settings);
		$emailBody = $emailView->render();
	
		return $emailBody;
	}
	
}